import nibabel as nb
import trimesh
import numpy as np
from slam import texture


def load_mesh(gifti_file):
    """
    load gifti_file and create a trimesh object
    :param gifti_file: str, path to the gifti file on the disk
    :return: the corresponding trimesh object
    """
    g = nb.gifti.read(gifti_file)
    coords, faces = g.getArraysFromIntent(
        nb.nifti1.intent_codes['NIFTI_INTENT_POINTSET'])[0].data, \
        g.getArraysFromIntent(
            nb.nifti1.intent_codes['NIFTI_INTENT_TRIANGLE'])[0].data
    metadata = g.get_meta().metadata
    metadata['filename'] = gifti_file
    return trimesh.Trimesh(faces=faces, vertices=coords,
                           metadata=metadata, process=False)


def write_mesh(mesh, gifti_file):
    """ Create a mesh object from two arrays

    fixme:  intent should be set !
    """
    coord = mesh.vertices
    triangles = mesh.faces
    carray = nb.gifti.GiftiDataArray().from_array(coord.astype(np.float32),
                                                  "NIFTI_INTENT_POINTSET")
    tarray = nb.gifti.GiftiDataArray().from_array(
        triangles.astype(np.float32), "NIFTI_INTENT_TRIANGLE")
    img = nb.gifti.GiftiImage(darrays=[carray, tarray])
    # , meta=mesh.metadata)

    nb.gifti.write(img, gifti_file)


def load_texture(gifti_file):
    """
    load gifti_file and create a TextureND object (multidimensional)
    :param gifti_file: str, path to the gifti file on the disk
    :return: the corresponding TextureND object
    """
    # read the gifti usinng nibabel
    nb_texture = nb.gifti.read(gifti_file)
    # concatenate all the data arrays in a single numpy array
    cat_darrays = [nb_texture.darrays[i].data
                   for i in range(len(nb_texture.darrays))]
    return texture.TextureND(darray=np.array(cat_darrays),
                             metadata=nb_texture.get_meta().metadata)


def write_texture(tex, gifti_file):
    """
    TODO manage metadata
    write a TextureND object to disk as a gifti file
    :param gifti_file: str, path to the gifti file on the disk
    :return: the corresponding TextureND object
    """
    darrays_list = []
    for d in tex.darray:
        gdarray = nb.gifti.GiftiDataArray().from_array(
            d.astype(np.float32), 0)
        # gdarray.metadata = tex.metadata
        # print(gdarray.metadata)
        darrays_list.append(gdarray)
    out_texture_gii = nb.gifti.GiftiImage(darrays=darrays_list)
    # out_metadata = tex.metadata
    # print(out_metadata)
    # out_metadata['Name']=gifti_file
    # print(out_metadata)
    # out_texture_gii.set_metadata(nb.gifti.GiftiMetaData(out_metadata))
    # , meta=str(tex.metadata))

    nb.gifti.write(out_texture_gii, gifti_file)



import slam.io as sio
import numpy as np
import unittest
import trimesh

# UTILITIES


def make_sphere_a():
    """ Create a sphere"""
    mesh_a = trimesh.creation.icosphere(subdivisions=1, radius=1.0)
    return mesh_a


class TestIOMethods(unittest.TestCase):

    # Sphere
    sphere_A = make_sphere_a()

    def test_basic(self):
        mesh_a = self.sphere_A.copy()
        mesh_a_save = self.sphere_A.copy()

        sio.write_mesh(mesh_a, "tests/data/io/temp.gii")

        # Non modification
        assert(mesh_a.vertices == mesh_a_save.vertices).all()
        assert(mesh_a.faces == mesh_a_save.faces).all()

        mesh_b = sio.load_mesh('tests/data/io/temp.gii')

        precision_A = .00001

        # Correctness
        assert(np.isclose(mesh_a.vertices, mesh_b.vertices, precision_A).all())
        assert(np.isclose(mesh_a.faces, mesh_b.faces, precision_A).all())


if __name__ == '__main__':
    unittest.main()



import numpy as np
import trimesh
import slam.differential_geometry as sdg
import slam.distortion as sdst
# from scipy import sparse as ssp
import slam.topology as stop
from scipy.sparse.linalg import lgmres
########################
# error tolerance for lgmres solver
solver_tolerance = 1e-6
########################


def spherical_mapping(mesh, mapping_type='laplacian_eigenvectors',
                      conformal_w=1, authalic_w=1, dt=0.01, nb_it=10):
    """
    ADD REF
    :param mesh:
    :param mapping_type:
    :param conformal_w:
    :param authalic_w:
    :param dt:
    :param nb_it:
    :return:
    """
    # computing spherical mapping based on laplacian eigenvectors
    sph_vert = sdg.mesh_laplacian_eigenvectors(mesh, nb_vectors=3)
    norm_sph_vert = np.sqrt(np.sum(sph_vert * sph_vert, 1))
    sphere_vertices = sph_vert / np.tile(norm_sph_vert, (3, 1)).T
    if mapping_type == 'laplacian_eigenvectors':
        return trimesh.Trimesh(faces=mesh.faces,
                               vertices=sphere_vertices,
                               metadata=mesh.metadata, process=False)

    if mapping_type == 'conformal':
        """
        Desbrun, M., Meyer, M., & Alliez, P. (2002).
        Intrinsic parameterizations of surface meshes.
        Computer Graphics Forum, 21(3), 209–218.
        https://doi.org/10.1111/1467-8659.00580
        """

        # options.symmetrize = 0;
        # options.normalize = 1;
        L, B = sdg.compute_mesh_laplacian(mesh, lap_type='conformal')

    if mapping_type == 'authalic':
        # options.symmetrize = 0;
        # options.normalize = 1;
        L, B = sdg.compute_mesh_laplacian(mesh, lap_type='authalic')

    if mapping_type == 'combined':
        # options.symmetrize = 0;
        # options.normalize = 1;
        Lconf, Bconf = sdg.compute_mesh_laplacian(mesh, lap_type='conformal')
        Laut, Baut = sdg.compute_mesh_laplacian(mesh, lap_type='authalic')
        L = conformal_w * Lconf + authalic_w * Laut
    # continue the spherical mappig by minimizig the energy
    evol = list()
    for it in range(nb_it):
        sphere_vertices = sphere_vertices - dt * L.dot(sphere_vertices)
        # sphere_vertices * L
        norm_sph_vert = np.sqrt(np.sum(sphere_vertices * sphere_vertices, 1))
        sphere_vertices = sphere_vertices / np.tile(norm_sph_vert, (3, 1)).T
        if it % 10 == 0:
            sph = trimesh.Trimesh(faces=mesh.faces,
                                  vertices=sphere_vertices,
                                  process=False)
            angle_diff = sdst.angle_difference(sph, mesh)
            area_diff = sdst.area_difference(sph, mesh)
            edge_diff = sdst.edge_length_difference(sph, mesh)
            evol.append([np.sum(np.abs(angle_diff.flatten())),
                         np.sum(np.abs(area_diff.flatten())),
                         np.sum(np.abs(edge_diff.flatten()))])

    # ind = 0;
    # for it=1:nb_it
    # % it = 1;
    # % while sum(I(: ) < 0)
    # % it = it + 1;
    # % vertex1 = vertex1 * L;
    # vertex1 = vertex1 - eta * vertex1 * L;
    # vertex1 = vertex1. / repmat(sqrt(sum(vertex1. ^ 2, 1)), [3 1]);
    # if mod(it, 100) == 0
    #     ind = ind + 1;
    #     NFV.vertices = vertex1
    #     ';
    #     [nb_inward, inward] = reversed_faces(NFV, 'sphere');
    #     bil_inward(ind) = nb_inward;
    #     w = zeros(1, m);
    #     E = zeros(1, m);
    #     for i=1:3
    #     i1 = mod(i, 3) + 1;
    #     % directed
    #     edge
    #     u = vertex1(:, faces(i,:)) - vertex1(:, faces(i1,:));
    #     % norm
    #     squared
    #     u = sum(u. ^ 2);
    #     % weights
    #     between
    #     the
    #     vertices
    #     for j=1:m
    #     w(j) = L(faces(i, j), faces(i1, j));
    # end
    # % w = W(faces(i,:) + (faces(i1,:) - 1)*n);
    # E = E + w. * u;
    #
    #
    # end
    #
    # bil_E(ind) = sum(E);
    # if ind > 2
    #     disp(['Ratio of inverted triangles:' num2str(100 * nb_inward / m, 3)
    #           '% energy decrease:' num2str(bil_E(end - 1) - bil_E(end), 3)]);
    # end
    # end
    # end

    return trimesh.Trimesh(faces=mesh.faces,
                           vertices=sphere_vertices,
                           metadata=mesh.metadata, process=False), evol


def disk_conformal_mapping(mesh, lap_type='conformal',
                           boundary=None, boundary_coords=None):
    """
    compute comformal mapping of a mesh to a disk
    ADD ref
    :param mesh: a trimesh object
    :param lap_type: type of mesh Laplacian to be used,
    see the function differential_geometry/compute_mesh_weights for more
    informations
    :param boundary: boundary of the mesh, resulting from the function
    topology/mesh_boundary
    :param boundary_coords: coordindates of the boundary vertices on the
    output disk, if None then uniform sampling
    :return: a trimesh object, planar disk representation of the input mesh
    """
    if boundary is None:
        boundary_t = stop.mesh_boundary(mesh)
        boundary = boundary_t[0]
    boundary = np.array(boundary)
    if boundary_coords is None:
        p = boundary.size
        t = np.arange(0, 2 * np.math.pi, (2 * np.math.pi / p))
        boundary_coords = np.array([np.cos(t), np.sin(t)])
    L, LB = sdg.compute_mesh_laplacian(mesh, lap_type=lap_type)
    Nv = len(mesh.vertices)  # np.array(mesh.vertex()).shape[0]
    print('Boundary Size:', boundary.shape)
    print('Laplacian Size:', L.shape)
    for i in boundary:
        L[i, :] = 0
        L[i, i] = 1
    L = L.tocsr()

    Rx = np.zeros(Nv)
    Ry = np.zeros(Nv)
    Rx[boundary] = boundary_coords[0, :]
    Ry[boundary] = boundary_coords[1, :]

    x, info = lgmres(L, Rx, tol=solver_tolerance)
    y, info = lgmres(L, Ry, tol=solver_tolerance)
    z = np.zeros(Nv)

    return trimesh.Trimesh(faces=mesh.faces,
                           vertices=np.array([x, y, z]).T,
                           metadata=mesh.metadata, process=False)


def moebius_transformation(a, b, c, d, plane_mesh):
    """
    see https://en.wikipedia.org/wiki/M%C3%B6bius_transformation
    :param a:Complex
    :param b:Complex
    :param c:Complex
    :param d:Complex
    :param plane_mesh: trimesh mesh
    :return:
    """
    array_complex = plane_mesh.vertices[:, 0] + \
        1.0j * plane_mesh.vertices[:, 1]
    numerator = (a * array_complex) + b
    denominator = (c * array_complex) + d

    transformed_complex_plane = numerator / denominator
    transformed_vertices = np.array([transformed_complex_plane.real,
                                     transformed_complex_plane.imag,
                                     plane_mesh.vertices[:, 2]]).T.copy()
    transformed_plane_mesh = \
        trimesh.Trimesh(vertices=transformed_vertices,
                        faces=plane_mesh.faces.copy(),
                        process=False)
    return transformed_plane_mesh


def stereo_projection(sphere_mesh, h=None, invert=True):
    """
    compute the stereographic projection from the unit sphere (center = 0,
    radius = 1) onto the horizontal plane which 3rd coordinate is h of the
    vertices given
    :param sphere_mesh: trimesh spherical mesh to be projected onto the plane
    :param h: 3rd coordinate of the projection plane
    :param invert: Boolean value to invert output mesh faces orientation
    upwards
    :return: trimesh planar mesh, 3rd coordinate is equal to h
    """
    vertices = sphere_mesh.vertices.copy()
    if h is None:
        h = -1
    for ind, vert in enumerate(vertices):
        vertices[ind, 0] = (-h + 1) * vert[0] / (1 - vert[2])
        vertices[ind, 1] = (-h + 1) * vert[1] / (1 - vert[2])
        vertices[ind, 2] = h

    plane_mesh = trimesh.Trimesh(vertices=vertices,
                                 faces=sphere_mesh.faces.copy(),
                                 process=False)
    if invert:
        plane_mesh.invert()
    return plane_mesh


def inverse_stereo_projection(plane_mesh, h=None, invert=True):
    """
    compute the inverse stereograhic projection from an horizontal plane onto
    the unit sphere (center = 0, radius = 1)
    :param plane_mesh: trimesh planar mesh to be inverse projected onto the
    sphere
    :param h: 3rd coordinate of the projection plane
    :param invert: Boolean value to invert input mesh faces orientation upwards
    to be consistent with stereo_projection
    :return: trimesh unit sphere from inverse projected plane_mesh
    """
    if invert:
        plane_mesh.invert()
    vertices = plane_mesh.vertices.copy()
    if h is None:
        h = vertices[0, 2]
    for ind, vert in enumerate(vertices):
        denom = ((1 - h) ** 2 + vert[0] ** 2 + vert[1] ** 2)
        vertices[ind, 2] = (-(1 - h) ** 2 + vert[0] ** 2 + vert[1] ** 2)\
            / denom
        vertices[ind, 1] = 2 * (1 - h) * vert[1] / denom
        vertices[ind, 0] = 2 * (1 - h) * vert[0] / denom

    sphere_mesh = trimesh.Trimesh(vertices=vertices,
                                  faces=plane_mesh.faces.copy(),
                                  process=False)
    return sphere_mesh



import slam.io as sio
import slam.topology as stop
import numpy as np
import unittest
import trimesh
import itertools
import random

# UTILITIES


def distinct_triangles(n=3):
    """ Create n distinct triangles """
    coords = []
    for i in range(n):
        coords.append([i, 1, 0])
        coords.append([i, 0, 0])
        coords.append([i, 0, 1])
    coords = np.array(coords)

    faces = []
    for i in range(n):
        faces.append([i * 3, i * 3 + 1, i * 3 + 2])
    faces = np.array(faces)

    return trimesh.Trimesh(faces=faces, vertices=coords, process=False)


def random_triangles(n=3):
    """ Create n triangles which can overlap """

    coords = []
    for i in range(n * 3):
        coords.append([random.random() * 5, random.random()
                       * 5, random.random() * 5])
    coords = np.array(coords)

    faces = []
    for i in range(n):
        faces.append([i * 3, i * 3 + 1, i * 3 + 2])
    faces = np.array(faces)

    return trimesh.Trimesh(faces=faces, vertices=coords, process=False)


def make_cut_sphere_a():
    """ Create a sphere cut by two parallel planes """
    mesh_a = trimesh.creation.icosphere(subdivisions=1, radius=1.0)

    mesh_a = trimesh.intersections.slice_mesh_plane(
        mesh_a, [
            0, 1, 0], [
            0, -.1, 0])
    mesh_a = trimesh.intersections.slice_mesh_plane(
        mesh_a, [0, -1, 0], [0, .9, 0])
    return mesh_a


def make_cut_sphere_b():
    """ Create a sphere cut by one parallel plane """
    mesh_a = trimesh.creation.icosphere(subdivisions=1, radius=1.0)

    mesh_a = trimesh.intersections.slice_mesh_plane(
        mesh_a, [
            0, 1, 0], [
            0, -.1, 0])
    return mesh_a


class TestTopologyMethods(unittest.TestCase):

    # Sphere cut by two planes, with new vertices added at the intersection
    cutSphere_A = make_cut_sphere_a()

    # Sphere cut by one plane, with new vertices added at the intersection
    cutSphere_B = make_cut_sphere_b()

    # Sphere cut by two planes, with no new vertices added at the intersection
    cutSphere_C = sio.load_mesh("tests/data/topology/mesh_C.gii")

    # A 3D disk with height 0 and radius 2
    disk_radius_2 = sio.load_mesh("tests/data/topology/mesh_D.gii")

    def test_boundaries_basic(self):
        mesh_a = self.cutSphere_A

        mesh_a_save = mesh_a.copy()

        boundary = stop.mesh_boundary(mesh_a)

        # Non modification
        assert(mesh_a.vertices == mesh_a_save.vertices).all()
        assert(mesh_a.faces == mesh_a_save.faces).all()

        # Correctness
        assert(len(boundary) == 2)

        # Type
        assert(isinstance(boundary, list))
        assert(isinstance(boundary[0], list))

        iterable = list(itertools.combinations(boundary, 2))

        # Uniqueness
        for b1, b2 in iterable:
            assert set(b1).intersection(set(b2)) == set()

    def test_boundaries_triangles(self):

        mesh_a = distinct_triangles(5)
        mesh_b = random_triangles(5)

        boundary_a = stop.mesh_boundary(mesh_a)
        boundary_b = stop.mesh_boundary(mesh_b)

        iterable_a = list(itertools.combinations(boundary_a, 2))
        iterable_b = list(itertools.combinations(boundary_b, 2))

        # Correctness
        assert(len(boundary_a) == 5)
        assert(len(boundary_b) == 5)

        # Uniqueness
        for b1, b2 in iterable_a:
            assert set(b1).intersection(set(b2)) == set()
        for b1, b2 in iterable_b:
            assert set(b1).intersection(set(b2)) == set()

    def test_angle_norm_boundary(self):

        # GET A DISK IN 3D

        radius = 2
        mesh_a = self.disk_radius_2
        coords3D = mesh_a.vertices

        # COMPUTE BOUNDARY

        boundary = stop.mesh_boundary(mesh_a)

        # Correctness
        assert(len(boundary) == 1)

        # COMPUTE ANGLE AND NORM VARIATIONS

        angles, norms = stop.boundary_angles(boundary[0], coords3D)
        angles = abs(180 - angles)

        sum_angles, sum_norms = sum(angles), sum(norms)

        # Computation on three arbitrary linked vertices
        three_angles, three_norms = stop.boundary_angles([0, 1, 2], coords3D)
        three_angles = abs(180 - three_angles)

        perimeter = radius * np.pi * 2
        precision_A = .001
        precision_B = .001

        # Coherence of the angle sum and norm sum
        assert(np.isclose(sum_angles, 360, precision_A))
        assert(np.isclose(sum_norms, perimeter, precision_B))

        # Coherence of a partial angle and a partial norm variation
        assert(np.isclose(three_angles[1], 360 / len(coords3D), precision_A))
        assert(
            np.isclose(
                three_norms[1],
                perimeter /
                len(coords3D),
                precision_B))

    def test_boundaries_intersection_copy(self):
        mesh_a = self.cutSphere_A

        boundary = stop.mesh_boundary(mesh_a)
        b1 = boundary[0]
        b2 = boundary[1]
        b2_save = b2.copy()

        concat = b1 + b2

        inter_1 = stop.boundaries_intersection([concat, b2])
        inter_1 = inter_1[0][2]

        # Non modification
        assert(b2 == b2_save)

        # Type
        assert (isinstance(inter_1, list))

        # Correctness
        assert(set(inter_1) == set(b2))

    def test_close_mesh(self):
        mesh_a = self.cutSphere_C.copy()

        boundary_prev = stop.mesh_boundary(mesh_a)

        # Correctness
        assert(len(boundary_prev) == 2)

        mesh_a_closed, nvertices_added = stop.close_mesh(mesh_a)

        # Coherence
        assert(len(mesh_a_closed.vertices) == len(
            mesh_a.vertices) + nvertices_added)

        boundary_closed = stop.mesh_boundary(mesh_a_closed)

        # Mesh is now closed
        assert(len(boundary_closed) == 0)

        # Non modification of the vertices which are not on the boundaries
        sbase = (set(range(len(mesh_a.vertices))))
        sbound1 = (set(boundary_prev[0]))
        sbound2 = (set(boundary_prev[1]))
        sbound = sbound1.union(sbound2)
        snot_bound = (sbase.difference(sbound))

        for i in snot_bound:
            assert (mesh_a.vertices[i] == mesh_a_closed.vertices[i]).all()

    def test_remove_mesh_boundary_faces(self):
        mesh_a = self.cutSphere_A.copy()
        mesh_a_save = mesh_a.copy()
        boundary = stop.mesh_boundary(mesh_a)
        mesh_processed = stop.remove_mesh_boundary_faces(mesh_a,
                                                         face_vertex_number=1)
        # Non modification
        assert(mesh_a.vertices == mesh_a_save.vertices).all()
        # Correctness
        # check that when removing all faces with any vertex on the boundary,
        # all the boundary vertices are removed from the mesh
        print(len(boundary[0]))
        print(len(mesh_a.vertices))
        print(len(mesh_processed.vertices))
        assert(len(mesh_processed.vertices) ==
               len(mesh_a.vertices) - len(boundary[0]))


if __name__ == '__main__':
    unittest.main()



import numpy as np


def angle(vec1, vec2):
    """
    Return the angle between two vectors
    :param vec1:
    :param vec2:
    :return:
    """
    return np.arccos(np.dot(vec1, vec2)/(np.sqrt(np.dot(vec1, vec1) *
                                                 np.dot(vec2, vec2))))


def dotprod(vec1, vec2):
    """
    Return the normalized dotprod between two vectors
    :param vec1:
    :param vec2:
    :return:
    """
    return np.dot(vec1, vec2)/(np.sqrt(np.dot(vec1, vec1)*np.dot(vec2, vec2)))


def compare_analytic_estimated_directions(analytic_directions,
                                          estimated_directions):
    """
    Compare the analytic principal directions with a estimated directions
    :param analytic_directions: (n,3) array
    :param estimated_directions: (n,3,2) array with the two directions
    that may be not necessarily ordered
    :return: angular error distribution
    """
    n = analytic_directions.shape[0]
    angular_error = np.zeros((n,))
    dotprods = np.zeros((n, 2))
    for i in range(n):
        angle0 = angle(analytic_directions[i, :],
                       estimated_directions[i, :, 0])
        angle1 = angle(analytic_directions[i, :],
                       estimated_directions[i, :, 1])
        angular_error[i] = min(np.mod(angle0, np.pi/4),
                               np.mod(angle1, np.pi/4))
        dotprods[i, 0] = dotprod(analytic_directions[i, :],
                                 estimated_directions[i, :, 0])
        dotprods[i, 1] = dotprod(analytic_directions[i, :],
                                 estimated_directions[i, :, 1])
    return angular_error, dotprods



import trimesh
import numpy as np


def save_image(scene_viewer, filename):
    """
    save a slam figure to disc
    :param scene:
    :param filename:
    :return:
    """
    scene = scene_viewer._scene
    with open(filename, 'wb') as f:
        f.write(scene.save_image(background=scene_viewer.background))


def visbrain_plot(mesh, tex=None, caption=None, cblabel=None, visb_sc=None,
                  cmap='jet'):
    """
    Visualize a trimesh object using visbrain core plotting tool
    :param mesh: trimesh object
    :param tex: numpy array of a texture to be visualized on the mesh
    :return:
    """
    from visbrain.objects import BrainObj, ColorbarObj, SceneObj
    b_obj = BrainObj('gui', vertices=np.array(mesh.vertices),
                     faces=np.array(mesh.faces),
                     translucent=False)
    if not isinstance(visb_sc, SceneObj):
        visb_sc = SceneObj(bgcolor='black', size=(1000, 1000))
    # identify (row, col)
    row, _ = get_visb_sc_shape(visb_sc)
    visb_sc.add_to_subplot(b_obj, row=row, col=0, title=caption)

    if tex is not None:
        b_obj.add_activation(data=tex, cmap=cmap,
                             clim=(np.min(tex), np.max(tex)))
        CBAR_STATE = dict(cbtxtsz=20, txtsz=20., width=.1, cbtxtsh=3.,
                          rect=(-.3, -2., 1., 4.), cblabel=cblabel)
        cbar = ColorbarObj(b_obj, **CBAR_STATE)
        visb_sc.add_to_subplot(cbar, row=row, col=1, width_max=200)

    return visb_sc


def get_visb_sc_shape(visb_sc):
    """
    get the subplot shape in a visbrain scene
    :param visb_sc:
    :return: tuple (number of rows, number of columns)
    """
    vb_shape = visb_sc._grid_desc.keys()
    if not len(vb_shape):
        rc = (0, 0)
    else:
        rc = (max([k[0] for k in vb_shape]), max([k[1] for k in vb_shape]))
    return rc


def pyglet_plot(mesh_in, values=None, color_map=None,
                plot_colormap=False, caption=None,
                alpha_transp=255, background_color=None,
                default_color=[100, 100, 100, 200], cmap_bounds=None):
    """
    Visualize a trimesh object using pyglet as proposed in trimesh
    the added value is for texture visualization
    :param mesh_in: trimesh object
    :param values: numpy array of a texture to be visualized on the mesh
    :param color_map: str, matplotlib colormap, default is 'jet'
    :param plot_colormap: Boolean, if True use matplotlib to plot the colorbar
     of the map on a separate figure
    :param caption: Title of window
    :param alpha_transp: mesh transparency parameter, 0=transparent, 255=solid
    :param background_color:
    :param default_color: color of vertices or faces where 'values' is NaN
    :param cmap_bounds : bounds to impose on the colormap
    :return:
    """
    import matplotlib.pyplot as plt
    # to ensure plotting do not affect the mesh (esp. visual aspects)
    mesh = mesh_in.copy()
    if background_color is not None:
        background = background_color
    else:
        # default background color is black
        background = [0, 0, 0, 255]
        # turn black the background of colormap fig
        plt.style.use('dark_background')
    fig = None
    smooth = True
    if values is None:
        mesh.visual.vertex_colors = default_color
    else:
        if color_map is None:
            color_map = plt.get_cmap('jet', 12)
        # in case NaN are present in 'values'
        nan_inds = np.isnan(values)
        if sum(nan_inds) == len(values):
            print('no value in the texture')
            vect_col_map = np.tile(default_color, (1, len(values)))
        else:

            vect_col_map_tmp = \
                trimesh.visual.color.interpolate(values[~nan_inds],
                                                 color_map=color_map)
            vect_col_map = np.zeros((len(values), 4))
            vect_col_map[~nan_inds, :] = vect_col_map_tmp
            vect_col_map[:, 3] = alpha_transp
            vect_col_map[nan_inds, :] = default_color

        if values.shape[0] == mesh.vertices.shape[0]:
            mesh.visual.vertex_colors = vect_col_map
        elif values.shape[0] == mesh.faces.shape[0]:
            mesh.visual.face_colors = vect_col_map
            smooth = False

        if plot_colormap:
            import matplotlib as mpl
            mpl.rcParams.update({'font.size': 20})
            # fig = plt.figure(figsize=(8, 2))
            # ax1 = fig.add_axes([0.05, 0.80, 0.9, 0.15])
            fig, ax = plt.subplots(1, 1)
            ax.set_title(caption)

            # if cmap_bounds is None:
            #     vmin = np.around(np.min(values[~nan_inds]))
            #     vmax = np.around(np.max(values[~nan_inds]))
            #     norm = mpl.colors.Normalize(vmin=vmin, vmax=vmax)
            # else:
            norm = mpl.colors.BoundaryNorm(cmap_bounds, color_map.N)
            # print(norm)
            mpl.colorbar.ColorbarBase(ax, cmap=color_map, norm=norm,
                                      orientation='horizontal')

            # mpl.colorbar.ColorbarBase(ax, cmap=color_map,
            #                          orientation='horizontal')
            fig.set_size_inches(18, 4)
            plt.show()

            # gradient = np.linspace(0, 1, 256)
            # gradient = np.vstack((gradient, gradient))
            # fig, ax = plt.subplots(1, 1)
            # # fig.subplots_adjust(top=0.95, bottom=0.05, left=0.01,
            # right=0.99)
            #
            # ax.set_title(caption)
            # ax.imshow(gradient, aspect='auto', cmap=color_map)
            # pos = list(ax.get_position().bounds)
            # y_text = pos[1] + pos[3] / 2.
            # fig.text(pos[0] - 0.01, y_text,
            #          '{:0.0000009f}'.format(np.min(values[~nan_inds])),
            #          va='center', ha='right', fontsize=15, color='k')
            # fig.text(pos[2] + pos[0] + 0.01, y_text,
            #          '{:0.0000009f}'.format(np.max(values[~nan_inds])),
            #          va='center', fontsize=15, color='k')
            # ax.set_axis_off()
            # fig.set_size_inches(18, 3)
            # plt.show()

    # call the default trimesh visualization tool using pyglet
    # light = trimesh.scene.lighting.DirectionalLight()
    # [trimesh.scene.lighting.Light]

    scene = trimesh.Scene(mesh)  # , lights=[light])
    # print(scene.graph)
    # #lights, transforms = lighting.autolight(self)
    # lights, transforms = trimesh.scene.lighting.autolight(scene)
    # # assign the transforms to the scene graph
    # for L, T in zip(lights, transforms):
    #     L.intensity = 10000
    #     scene.graph[L.name] = T
    #     # set the lights
    # scene._lights = lights
    scene_viewer = scene.show(caption=caption,
                              smooth=smooth,
                              background=background)
    if fig is None:
        output = [scene_viewer]
    else:
        output = [scene_viewer, fig]
    return output



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

BASE_REQUIREMENTS = ['numpy', 'scipy', 'trimesh', 'nibabel', 'networkx']
TEST_REQUIREMENTS = ['flake8', 'autopep8', 'pytest', 'pytest-cov', 'coveralls']
DIST = ['tvb-gdist']
VISU = ['visbrain']

setup(
    name="brain-slam",
    version='0.0.2',
    packages=find_packages(),
    author="Guillaume Auzias",
    description="Surface anaLysis And Modeling",
    url="https://github.com/gauzias/slam",
    license='MIT',
    python_requires='>=3.6',  # enforce Python 3.6 as minimum
    install_requires=BASE_REQUIREMENTS,
    extras_require={
        'default-dev': TEST_REQUIREMENTS,
        'advanced-user': DIST,
        'advanced-dev':  DIST + TEST_REQUIREMENTS,
        'full': DIST + TEST_REQUIREMENTS + VISU
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix", "Operating System :: MacOS :: MacOS X"
    ],


)

